<?php
	/**
	* 7-PDF PHP Library CLASS
	*
	* Date: 07.01.2010<br/>
	* <br/>
	* This PHP Class makes it possible to connect directly to a 7-PDF Api Service.<br/>
	* Currently this PHP Class supports 84 file formats that can be directly converted to PDF.<br/>
	* It gives you the possibility to create PDF-on-the-fly by using a powerfull client/server based conversion service.<br/>
	* <br/>
	* Define multiple pdf settings, or set some pdf file encryptions or permissions by php source and convert to PDF on-the-fly!<br/>
	* The connection to a 7-PDF Api Service is possible over web or intranet by using the HTTP-Protocol and totally free! 
	* For this, you have to do a small registration to get access to the free pdf converter webservice at http://api.7-pdf.com/1.0/Converter<br/>
	* The registration protects your documents with your choosen personal username and password against unauthorized access.
	* <br/>
	* <br/>
	* @author Thorsten Hodes <hodes-software@gmx.de>
	* @copyright Copyright (C) 2010 Thorsten Hodes
	* @copyright Portions Copyright (C) 2007 Free Software Foundation, Inc. http://www.fsf.org/
	* @license http://www.fsf.org/licensing/licenses/gpl.html GNU General Public License
	* @Version 1.0
	*/
	
	/**
	* Include HTTP Client Class
	*/
	include "PEAR/Net/HTTP/class.net_http_client.inc.php";
	
	/**
	* Include the necessary 7-PDF Api Type-Classes
	**/
	
	include "class.sevenpdf_in_object_type.inc.php";
	include "class.sevenpdf_options_type.inc.php";
	include "class.sevenpdf_out_object_type.inc.php";
	
	/**
	* Debug level - debug methods calls
	*/
	define( "DBGTRACE", 1 );
	/**
	* Debug level - debug data received
	*/
	define( "DBGINDATA", 2 );
	/**
	* Debug level - debug data sent
	*/
	define( "DBGOUTDATA", 4 );
	/**
	* Debug level - debug low-level (usually internal) methods
	*/
	define( "DBGLOW", 8 );
	/**
	* Debug level - debug socket-level code
	*/
	define( "DBGSOCK", 16 );
	/**
	* Internal error: connection failed
	*/
	define( "ECONNECTION", -1 );
	/**
	* Internal error: response status line is not http compliant
	*/
	define( "EBADRESPONSE", -2 );

	
	/**
	* Set Script Timeout
	*/
	SET_TIME_LIMIT(180); //3min.
	

	/**
	* 7-PDF PHP Library CLASS
	* 
	* 
	*/
	
	class sevenpdf 
	{
		// @private
		
		//http object Net_HTTP_Client
		var $http;

		//WSDL Specification of 7-PDF API Service
		var $wsdl;
		
		//Specified the Service - URL Location
		var $service_url;
		
		//Options Array for PDF-Settings
		var $options;	
		
		//The Service-User that connects to the 7-PDF Remote Api Service
		var $username;
		
		//The Service-Password for connection to the 7-PDF Remote Api Service
		var $password;
		
		/**
		* Supported filetype array of the sourcefile
		*/
		var $supported_file_types_array = array(	
																
																'doc','dot','xls','xlt','ppt','pps','pot',
																'odt','ott','ods','ots','odp','otp','sxw',
																'stw','sxc','stc','swi','sti','sdw','vor',
																'sdc','sdd','rtf','txt','csv','html','htm',
																'xml','dif','dxf','eps','tif','odg','otg',
																'sxd','std','jpg','jpeg','png','gif','bmp',
																'odf','sxm','mml','smf','docx','docm','dotx',
																'dotm','xlsx','xlsb','xlsm','xltm','xltx','pptx',
																'pptm','potm','odm','sxg','sgl','slk','wpd',
																'oth','wb2','sda','emf','met','pct','sgf',
																'sgv','svm','wmf','pbm','pcx','pgm','ppm',
																'psd','ras','tga','tiff','xbm','xpm','pcd'
																
															);
		
		
		/**
		* sevenpdf
		* CONSTRUCTOR
		* @param host The Host of the 7-PDF Api Service.<br/>For example localhost, www.your-domain.de, 192.168.1.100)
		* @param port The port, where the service is running on the host.
		* @param ssl Enables SSL-Support if necessary.
		* @param username The Service-User that is necessary to connect the 7-PDF Remote Api Service.
		* @param password The Service-Password that is necessary to connect the 7-PDF Remote Api Service.
		**/	
		function __construct( $host= NULL, $username= NULL, $password, $ssl=False, $port= NULL )
		{
			if( $this->debug & DBGTRACE ) echo "__construct( $host, $username, $password, $ssl, $port )\n";
			
			if( $host != NULL ) {
				$this->http = new Net_HTTP_Client($host,$port,$ssl);	
				
				if($ssl){
					$this->wsdl = "https://$host/1.0/Converter?WSDL";
					$this->service_url = "https://$host";
				}
				else{
					if($port != 80){
						$this->wsdl = "http://$host:$port/1.0/Converter?WSDL";
						$this->service_url = "http://$host:$port";
					}
				 	else{
						$this->wsdl = "http://$host/1.0/Converter?WSDL";
						$this->service_url = "http://$host";
					}
				}
				
			}
			
			//Sets the authorizations to webservice
			if( $username != NULL ) {
				$this->http->setCredentials( $username, $password );
				$this->username = $username;
				$this->password = $password;
			}
			
			//******************************************
			//* Init PDF Default-Setting Object 
			//******************************************
			
			$this->options[17] = new sevenpdf_options_type();
			$this->options[0]->name = 'UseTaggedPDF';
			$this->options[0]->value = 'False';
			$this->options[1]->name = 'ExportNotes';
			$this->options[1]->value = 'True';
			$this->options[2]->name = 'ExportNotesPages';
			$this->options[2]->value = 'True';
			$this->options[3]->name = 'ExportBookmarks';
			$this->options[3]->value = 'True';
			$this->options[4]->name = 'SelectPDFVersion';
			$this->options[4]->value = '0';
			$this->options[5]->name = 'PageRange';
			$this->options[5]->value = '';
			$this->options[6]->name = 'UseLosslessCompression';
			$this->options[6]->value = 'True';
			$this->options[7]->name = 'Quality';
			$this->options[7]->value = '75';
			$this->options[8]->name = 'ReduceImageResolution';
			$this->options[8]->value = 'False';
			$this->options[9]->name = 'MaxImageResolution';
			$this->options[9]->value = '300';
			$this->options[10]->name = 'EncryptFile';
			$this->options[10]->value = 'False';
			$this->options[11]->name = 'DocumentOpenPassword';
			$this->options[11]->value = '';
			$this->options[12]->name = 'RestrictPermissions';
			$this->options[12]->value = 'False';
			$this->options[13]->name = 'PermissionPassword';
			$this->options[13]->value = '';
			$this->options[14]->name = 'Printing';
			$this->options[14]->value = '2';
			$this->options[15]->name = 'Changes';
			$this->options[15]->value = '4';
			$this->options[16]->name = 'EnableCopyingOfContent';
			$this->options[16]->value = 'True';
			$this->options[17]->name = 'EnableTextAccessForAccessibilityTools';
			$this->options[17]->value = 'True';
		}
		
		/**
		* sevenpdf
		* DESTRUCTOR
		**/
		
		function __destruct()
		{
			$this->http->Disconnect();
			$this->http = NULL;
			$this->options = NULL;
		}
		
		/**
	 	* rand
		* Returns mt_rand for min/max
		* @param min Miniumum Value for randomization
		* @param max Maximum Value for randomazation
		* @return mt_rand value
		**/
		function rand($min = null, $max = null) 
		{
			static $seeded;

			if (!isset($seeded)) {
				mt_srand((double)microtime()*1000000);
				$seeded = true;
			}

			if (isset($min) && isset($max)) {
				if ($min >= $max) {
					return $min;
				} else {
				return mt_rand($min, $max);
			}
			} else {
				return mt_rand();
			}
		}
		
		/**
	 	* create_random_value
		* Returns a randomized string
		* @param length Defines the randomized string-length
		* @param type Defines possible digit values
		* @return string randomized
		**/
		function create_random_value($length, $type = 'mixed') 
		{
			if ( ($type != 'mixed') && ($type != 'chars') && ($type != 'digits')) return false;

			$rand_value = '';
			while (strlen($rand_value) < $length) {
				if ($type == 'digits') {
					$char = $this->rand(0,9);
				} else {
					$char = chr($this->rand(0,255));
				}
				if ($type == 'mixed') {
					if (eregi('^[a-z0-9]$', $char)) $rand_value .= $char;
				} elseif ($type == 'chars') {
					if (eregi('^[a-z]$', $char)) $rand_value .= $char;
				} elseif ($type == 'digits') {
					if (ereg('^[0-9]$', $char)) $rand_value .= $char;
				}
			}

			return $rand_value;
		}
		
				
		/**
	 	* getRemoteFolderName
		* Returns a remote folder name for the pdf creation process
		* @return string timestamp as folder name
		**/
		function getRemoteFolderName()
		{
			//Return the unix timestamp as unique folder name
			return $this->create_random_value(12);
		}
		
		/**
	 	* getFileType
		* Returns the filetype extension of a file
		* @param file string path of file
		* @return string filetype extension (txt, doc, etc.)
		**/
		function getFileType( $file )
		{
			return strtolower(end(explode(".", $file)));	
		}
				
		/**
	 	* getIsSupportedFileType
		* Returns true if sourcefile type is supported
		* @param file string path of the source file
		* @return string timestamp as folder name
		**/
		function getIsSupportedFileType( $file )
		{
			
			$filetype = $this->getFileType($file);
			$supported = false;
			
			foreach($this->supported_file_types_array as $supported_files_type_value)
			{

    			if ($filetype == $supported_files_type_value)
    			{
        			$supported = true;
    			}
    			
			}
			
			return $supported;
		}
		
		
		/**
		* setUseTaggedPDFOption
		* Sets the PDFOption UseTaggedPDF to generate tagged PDF Files
		* @param value The value can be true or false (default).
		**/
		function setUseTaggedPDFOption( $value )
		{
			if($value){
				$this->options[0]->value = "True";		
			}
			else{
				$this->options[0]->value = "False";
			}
			
		}
	
		/**
		* setExportNotesPDFOption
		* Exports notes to the generated PDF Files
		* @param value The value can be true or false (default).
		**/
		function setExportNotesPDFOption( $value )
		{
			if($value){
				$this->options[1]->value = "True";		
			}
			else{
				$this->options[1]->value = "False";
			}
			
		}
		
		/**
		* setExportNotesPagesPDFOption
		* Exports notes pages to generate PDF Files
		* @param value The value can be true (default) or false.
		**/
		function setExportNotesPagesPDFOption( $value )
		{
			if($value){
				$this->options[2]->value = "True";		
			}
			else{
				$this->options[2]->value = "False";
			}
			
		}
		
		/**
		* setExportBookmarksPDFOption
		* Exports bookmarks to generate PDF Files
		* @param value The value can be true (default) or false.
		**/
		function setExportBookmarksPDFOption( $value )
		{
			if($value){
				$this->options[3]->value = "True";		
			}
			else{
				$this->options[3]->value = "False";
			}
			
		}
		
		/**
		* setSelectPDFVersionPDFOption
		* Sets the pdf version that should be generated
		* @param value The value can be 0 (PDF 1.4, default) or 1 (PDF/A-1). 
		**/
		function setSelectedPDFVersionPDFOption( $value )
		{
			if ($value >= 0 && $value <= 1){
				$this->options[4]->value = $value;
			}
		}
		
		/**
		* setPageRangePDFOption
		* Sets the page range for pdf generation
		* @param value The value can be a valid PageRange string like 1; 1-2; <br/>e.g. or you leave it empty for all pages.
		**/
		function setPageRangePDFOption( $value )
		{
			$this->options[5]->value = $value;
		}
		
		/**
		* setUseLosslessCompressionPDFOption
		* Sets a lossless compression level for images inside the generated PDF
		* @param value This compression state can be true (default) or false.
		**/
		function setUseLosslessCompressionPDFOption( $value )
		{
			if($value){
				$this->options[6]->value = "True";		
			}
			else{
				$this->options[6]->value = "False";
			}	
		}
		
		/** 
		* setImageQualityPDFOption
		* Sets the quality value in percent (0 - 100) for image compressions
		* inside the generated PDF.
		* @param value Possible value range from 0 - 100 (percent). Default = 50
		**/
		function setImageQualityPDFOption( $value )
		{
			if ($value >= 0 && $value <= 100){
				$this->options[7]->value = $value;
			}	
		}
		
		/**
		* setReduceImageResolutionPDFOption
		* Reduces the image resolution level of images
		* @param value The ReduceImageResolution state can be true or false (default).
		**/
		function setReduceImageResolutionPDFOption( $value )
		{
			if($value){
				$this->options[8]->value = "True";		
			}
			else{
				$this->options[8]->value = "False";
			}	
		}
		
		/**
		* setMaxImageResolutionPDFOption
		* The value of the maximum allowed image resolution
		* Possible values (DPI) are 75 (default), 150, 300, 600, 1200
		* Notice: The PDF Option ReduceImageResolution must be set to true to enable this option!
		* @param value Should be a possible value that stands for a DPI resolution of images inside the PDF.
		* @see setReduceImageResolutionPDFOption()
		**/
		function setMaxImageResolutionPDFOption( $value )
		{
			switch($value){
				case 75:
					$this->options[9]->value = "75";
					break;
				case 150:
					$this->options[9]->value = "150";
					break;
				case 300:
					$this->options[9]->value = "300";
					break;			
				case 600:
					$this->options[9]->value = "600";
					break;			
				case 1200:
					$this->options[9]->value = "1200";
					break;			
					
			}	
		}
		
		/**
		* setEncryptFilePDFOption
		* Enables the PDF File Encryption for generated PDF Files
		* @param value The encryption state can be true (encrypt files) or false (no encryption, default).
		**/
		function setEncryptFilePDFOption( $value )
		{
			if($value){
				$this->options[10]->value = "True";		
			}
			else{
				$this->options[10]->value = "False";
			}		
		}
		
		/**
		* setOpenPasswordPDFOption
		* Password that must be known when the generated PDF will be opened.
		* So the generated PDF File is password protected
		* Notice: setEncryptFilePDFOption() method must be set to true.
		* @param open_password The value can be any password string that you would like to use for your pdf encryption.
		* @see setEncryptFilePDFOption( $value )
		**/
		function setOpenPasswordPDFOption( $open_password )
		{
			$this->options[11]->value = $open_password;
		} 
		
		/**
		* setRestrictPermissionsPDFOption
		* Enables the encryption level to restrict pdf permissions
		* @param value The restrict permission state can be true (pdf with permission settings) or false (pdf without permission settings, default).
		**/
		function setRestrictPermissionsPDFOption( $value )
		{
			if($value){
				$this->options[12]->value = "True";		
			}
			else{
				$this->options[12]->value = "False";
			}
		}
	
		/**
		* setPermissionPasswordPDFOption
		* Password that protects the pdf permission level for generated PDF Files.
		* Notice: setRestrictPermissionsPDFOption method must be set to true.
		* @param permission_password The value can be any password string that you would like to use for your pdf encryption.
		* @see setEncryptFilePDFOption( $value )
		**/
		function setPermissionPasswordPDFOption( $permission_password )
		{
			$this->options[13]->value = $permission_password;
		} 
		
		/**
		* setPermissionPrintingPDFOption
		* Set permissions for printing level of the generated PDF File.
		* Possible is -> 0: Not permitted, 1: Low resolution (150 DPI), 2: High resolution (default)
		* Notice: setRestrictPermissionsPDFOption method must be set to true and
		* setPermissionPasswordPDFOption must be set with a permission password
		* @param value The permission value of printing (0,1,2)
		* @see setRestrictPermissionsPDFOption( $value ), setPermissionPasswordPDFOption( $permission_password )
		**/
		function setPermissionPrintingPDFOption( $value )
		{
			switch($value){
				case 0:
					$this->options[14]->value = "0"; //Not permitted
					break;
				case 1:
					$this->options[14]->value = "1"; //Low resolution (150 DPI)
					break;	
				case 2:
					$this->options[14]->value = "2"; //High resolution (default)
					break;
			}
		}
		
		/**
		* setPermissionChangesPDFOption
		* Set permissions for changes for the generated PDF File.
		* Possible is -> 0: Not permitted, 1: Inserting, deleting and rotating pages, 2: Filling in form fields, 
		* 3: Commenting, filling in form fields, 4: Any except extracting pages (default)
		* Notice: setRestrictPermissionsPDFOption method must be set to true and
		* setPermissionPasswordPDFOption must be set with a permission password
		* @param value The permission value of printing (0,1,2,3,4)
		* @see setRestrictPermissionsPDFOption( $value ), setPermissionPasswordPDFOption( $permission_password )
		**/
		function setPermissionChangesPDFOption( $value )
		{
			switch($value){
				case 0:
					$this->options[15]->value = "0"; //Not permitted
					break;
				case 1:
					$this->options[15]->value = "1"; //Inserting, deleting and rotating pages
					break;	
				case 2:
					$this->options[15]->value = "2"; //Filling in form fields
					break;
				case 3:
					$this->options[15]->value = "3"; //Commenting, filling in form fields
					break;
				case 4:
					$this->options[15]->value = "4"; //Any except extracting pages (default)
					break;
			}
		}
		
		/**
		* setEnableCopyingOfContentPDFOption
		* If set a pdf reader of the PDF File can extract and copy included pdf content 
		* @param value This permission value can be true (default) or false;
		**/
		function setEnableCopyingOfContentPDFOption( $value )
		{
			if($value){
				$this->options[16]->value = "True";		
			}
			else{
				$this->options[16]->value = "False";
			}
		}
		
		/**
		* setEnableTextAccessForAccessibilityToolsPDFOption
		* Accessibility Tools have access to the pdf text.
		* @param value This permission value can be true (default) or false;
		**/
		function setEnableTextAccessForAccessibilityToolsPDFOption( $value )
		{
			if($value){
				$this->options[17]->value = "True";		
			}
			else{
				$this->options[17]->value = "False";
			}
		}	
	
		/**
		* runConvertToPDF
		* Starts a conversion run.
		* @param source_file The file that you would like to convert to PDF.
		* You can use relative and absolute filepaths like files/letter.doc or /usr/local/files/letter.doc
		* @param PDFFile The result pdf.
		* @return True if conversion run was successfull, false if not.
		*/
		function runConvertToPDF($source_file, $pdf_file)
		{

			if ($this->username != NULL && $this->getIsSupportedFileType($source_file)){
				
				try{
					
					$remotepath = "/webdav/".$this->username."/".$this->getRemoteFolderName()."/";
					$file_remothepath = $remotepath."infile.".$this->getFileType($source_file);
					
					$this->http->MkCol($remotepath);
					$this->http->Put($file_remothepath, file_get_contents($source_file));
			
					$InObject = new sevenpdf_in_object_type();
					$InObject->option = $this->options;
					$InObject->sourceURL = $file_remothepath;
			
					/* Call Webservice now */
					$classmap = array('IN_Object' => 'ComplexType_IN_Object');
					$client = new SoapClient($this->wsdl, array('classmap' => $classmap));
			
					$OutObject = new sevenpdf_out_object_type();
					$OutObject = $client->convertLAN($InObject);
			
					if( $this->http->Get( $OutObject->resultURL ) == 200 ){	
						
						//7-PDF Service Authorization Context
						$context = stream_context_create(
																		array(
    																		'http' => array(
        																					'header'  => "Authorization: Basic " . base64_encode("$this->username:$this->password")
    																					)
    																	)
    																);

						file_put_contents($pdf_file, file_get_contents($this->service_url.$OutObject->resultURL, false, $context));
						
						$InObject = null;
						$OutObject = null;
						$client = null;
						
						return true;	
					}
					else{
						
						echo "Error: Conversion to PDF failed.\n\r";
						echo "Please retry or contact 7-pdf eMail-Support: h o d e s - s o f t w a r e [AT] g m x . d e\n\r";
						echo "Please inform us about your current username and error details for help.\n\r";
						
						return false;
					}
			
				} catch (Exception $e) {
					echo "Exception-Error Caught: ".$e."\n\r";
				}	
			}
			else{
				
				echo "Information: Username is empty or filetype of sourcefile seems to be an unsupported one.\n\r";
						
				return false;
			}
		}
	}	
?>